@@ -1,13 +1,12 @@
-from typing import Callable
-
 from prompt_toolkit.application import get_app
 from prompt_toolkit.enums import DEFAULT_BUFFER
-from prompt_toolkit.filters import Condition
+from prompt_toolkit.filters import Condition, Filter
 
 from mycli.packages.special import iocommands
+from mycli.packages.special.main import COMMANDS as SPECIAL_COMMANDS
 
 
-def cli_is_multiline(mycli) -> Callable:
+def cli_is_multiline(mycli) -> Filter:
     @Condition
     def cond():
         doc = get_app().layout.get_buffer_by_name(DEFAULT_BUFFER).document
@@ -23,38 +22,34 @@ def cli_is_multiline(mycli) -> Callable:
 def _multiline_exception(text: str) -> bool:
     orig = text
     text = text.strip()
+    first_word = text.split(' ')[0]
 
     # Multi-statement favorite query is a special case. Because there will
     # be a semicolon separating statements, we can't consider semicolon an
     # EOL. Let's consider an empty line an EOL instead.
-    if text.startswith("\\fs"):
+    if first_word.startswith("\\fs"):
         return orig.endswith("\n")
 
     return (
         # Special Command
-        text.startswith("\\")
-        or
-        # Delimiter declaration
-        text.lower().startswith("delimiter")
-        or
-        # Ended with the current delimiter (usually a semi-column)
-        text.endswith((
+        first_word.startswith("\\")
+        or text.endswith((
+            # Ended with the current delimiter (usually a semi-column)
             iocommands.get_current_delimiter(),
+            # or ended with certain commands
             "\\g",
             "\\G",
             r"\e",
+            r"\edit",
             r"\clip",
         ))
         or
-        # Exit doesn't need semi-column`
-        (text == "exit")
+        # non-backslashed special commands such as "exit" or "help" don't need semicolon
+        first_word in SPECIAL_COMMANDS
         or
-        # Quit doesn't need semi-column
-        (text == "quit")
-        or
-        # To all teh vim fans out there
-        (text == ":q")
+        # uppercase variants accepted
+        first_word.lower() in SPECIAL_COMMANDS
         or
         # just a plain enter without any text
-        (text == "")
+        (first_word == "")
     )
