@@ -1,32 +1,101 @@
 import logging
+import webbrowser
 
+import prompt_toolkit
+from prompt_toolkit.application.current import get_app
 from prompt_toolkit.enums import EditingMode
-from prompt_toolkit.filters import completion_is_selected, emacs_mode
+from prompt_toolkit.filters import (
+    Condition,
+    completion_is_selected,
+    control_is_searchable,
+    emacs_mode,
+)
 from prompt_toolkit.key_binding import KeyBindings
 from prompt_toolkit.key_binding.key_processor import KeyPressEvent
 
+from mycli.constants import DOCS_URL
 from mycli.packages import shortcuts
 from mycli.packages.toolkit.fzf import search_history
+from mycli.packages.toolkit.utils import safe_invalidate_display
 
 _logger = logging.getLogger(__name__)
 
 
+@Condition
+def ctrl_d_condition() -> bool:
+    """Ctrl-D exit binding is only active when the buffer is empty."""
+    app = get_app()
+    return not app.current_buffer.text
+
+
+@Condition
+def in_completion() -> bool:
+    app = get_app()
+    return bool(app.current_buffer.complete_state)
+
+
+def print_f1_help():
+    app = get_app()
+    app.print_text('\n')
+    app.print_text([
+        ('', 'Inline help — type "'),
+        ('bold', 'help'),
+        ('', '" or "'),
+        ('bold', r'\?'),
+        ('', '"\n'),
+    ])
+    app.print_text([
+        ('', 'Docs index — '),
+        ('bold', DOCS_URL),
+        ('', '\n'),
+    ])
+    app.print_text('\n')
+
+
 def mycli_bindings(mycli) -> KeyBindings:
     """Custom key bindings for mycli."""
     kb = KeyBindings()
 
+    @kb.add('f1')
+    def _(event: KeyPressEvent) -> None:
+        """Open browser to documentation index."""
+        _logger.debug('Detected F1 key.')
+        webbrowser.open_new_tab(DOCS_URL)
+        prompt_toolkit.application.run_in_terminal(print_f1_help)
+        safe_invalidate_display(event.app)
+
+    @kb.add('escape', '[', 'P')
+    def _(event: KeyPressEvent) -> None:
+        """Open browser to documentation index."""
+        _logger.debug("Detected alternate F1 key sequence.")
+        webbrowser.open_new_tab(DOCS_URL)
+        prompt_toolkit.application.run_in_terminal(print_f1_help)
+        safe_invalidate_display(event.app)
+
     @kb.add("f2")
     def _(_event: KeyPressEvent) -> None:
         """Enable/Disable SmartCompletion Mode."""
         _logger.debug("Detected F2 key.")
         mycli.completer.smart_completion = not mycli.completer.smart_completion
 
+    @kb.add('escape', '[', 'Q')
+    def _(_event: KeyPressEvent) -> None:
+        """Enable/Disable SmartCompletion Mode."""
+        _logger.debug("Detected alternate F2 key sequence.")
+        mycli.completer.smart_completion = not mycli.completer.smart_completion
+
     @kb.add("f3")
     def _(_event: KeyPressEvent) -> None:
         """Enable/Disable Multiline Mode."""
         _logger.debug("Detected F3 key.")
         mycli.multi_line = not mycli.multi_line
 
+    @kb.add('escape', '[', 'R')
+    def _(_event: KeyPressEvent) -> None:
+        """Enable/Disable Multiline Mode."""
+        _logger.debug('Detected alternate F3 key sequence.')
+        mycli.multi_line = not mycli.multi_line
+
     @kb.add("f4")
     def _(event: KeyPressEvent) -> None:
         """Toggle between Vi and Emacs mode."""
@@ -38,6 +107,17 @@ def mycli_bindings(mycli) -> KeyBindings:
             event.app.editing_mode = EditingMode.VI
             mycli.key_bindings = "vi"
 
+    @kb.add('escape', '[', 'S')
+    def _(event: KeyPressEvent) -> None:
+        """Toggle between Vi and Emacs mode."""
+        _logger.debug('Detected alternate F4 key sequence.')
+        if mycli.key_bindings == 'vi':
+            event.app.editing_mode = EditingMode.EMACS
+            mycli.key_bindings = 'emacs'
+        else:
+            event.app.editing_mode = EditingMode.VI
+            mycli.key_bindings = 'vi'
+
     @kb.add("tab")
     def _(event: KeyPressEvent) -> None:
         """Force autocompletion at cursor."""
@@ -48,6 +128,16 @@ def mycli_bindings(mycli) -> KeyBindings:
         else:
             b.start_completion(select_first=True)
 
+    @kb.add("escape", eager=True, filter=in_completion)
+    def _(event: KeyPressEvent) -> None:
+        """Cancel completion menu.
+
+        There will be a lag when canceling Escape due to the processing of
+        Alt- keystrokes as Escape- sequences.
+
+        There will be no lag when using control-g to cancel."""
+        event.app.current_buffer.cancel_completion()
+
     @kb.add("c-space")
     def _(event: KeyPressEvent) -> None:
         """
@@ -76,14 +166,8 @@ def mycli_bindings(mycli) -> KeyBindings:
         _logger.debug("Detected <C-x p>/> key.")
 
         b = event.app.current_buffer
-        cursorpos_relative = b.cursor_position / max(1, len(b.text))
-        pretty_text = mycli.handle_prettify_binding(b.text)
-        if len(pretty_text) > 0:
-            b.text = pretty_text
-            cursorpos_abs = int(round(cursorpos_relative * len(b.text)))
-            while 0 < cursorpos_abs < len(b.text) and b.text[cursorpos_abs] in (" ", "\n"):
-                cursorpos_abs -= 1
-            b.cursor_position = min(cursorpos_abs, len(b.text))
+        if b.text:
+            b.transform_region(0, len(b.text), mycli.handle_prettify_binding)
 
     @kb.add("c-x", "u", filter=emacs_mode)
     def _(event: KeyPressEvent) -> None:
@@ -95,14 +179,8 @@ def mycli_bindings(mycli) -> KeyBindings:
         _logger.debug("Detected <C-x u>/< key.")
 
         b = event.app.current_buffer
-        cursorpos_relative = b.cursor_position / max(1, len(b.text))
-        unpretty_text = mycli.handle_unprettify_binding(b.text)
-        if len(unpretty_text) > 0:
-            b.text = unpretty_text
-            cursorpos_abs = int(round(cursorpos_relative * len(b.text)))
-            while 0 < cursorpos_abs < len(b.text) and b.text[cursorpos_abs] in (" ", "\n"):
-                cursorpos_abs -= 1
-            b.cursor_position = min(cursorpos_abs, len(b.text))
+        if b.text:
+            b.transform_region(0, len(b.text), mycli.handle_unprettify_binding)
 
     @kb.add("c-o", "d", filter=emacs_mode)
     def _(event: KeyPressEvent) -> None:
@@ -140,7 +218,7 @@ def mycli_bindings(mycli) -> KeyBindings:
 
         event.app.current_buffer.insert_text(shortcuts.server_datetime(mycli.sqlexecute, quoted=True))
 
-    @kb.add("c-r", filter=emacs_mode)
+    @kb.add("c-r", filter=control_is_searchable)
     def _(event: KeyPressEvent) -> None:
         """Search history using fzf or reverse incremental search."""
         _logger.debug("Detected <C-r> key.")
@@ -148,13 +226,31 @@ def mycli_bindings(mycli) -> KeyBindings:
         if mode == 'reverse_isearch':
             search_history(event, incremental=True)
         else:
-            search_history(event)
+            search_history(
+                event,
+                highlight_preview=mycli.highlight_preview,
+                highlight_style=mycli.syntax_style,
+            )
 
-    @kb.add("escape", "r", filter=emacs_mode)
+    @kb.add("escape", "r", filter=control_is_searchable & emacs_mode)
     def _(event: KeyPressEvent) -> None:
         """Search history using fzf when available."""
         _logger.debug("Detected <alt-r> key.")
-        search_history(event)
+        search_history(
+            event,
+            highlight_preview=mycli.highlight_preview,
+            highlight_style=mycli.syntax_style,
+        )
+
+    @kb.add('c-d', filter=ctrl_d_condition)
+    def _(event: KeyPressEvent) -> None:
+        """Exit mycli or ignore keypress."""
+        _logger.debug('Detected <C-d> key on empty line.')
+        mode = mycli.config.get('keys', {}).get('control_d', 'exit')
+        if mode == 'exit':
+            event.app.exit(exception=EOFError, style='class:exiting')
+        else:
+            event.app.output.bell()
 
     @kb.add("enter", filter=completion_is_selected)
     def _(event: KeyPressEvent) -> None:
