@@ -11,6 +11,7 @@ from typing import Any, Generator
 
 import click
 from configobj import ConfigObj
+from prompt_toolkit.formatted_text import ANSI, FormattedText, to_plain_text
 from pymysql.cursors import Cursor
 import pyperclip
 import sqlparse
@@ -19,8 +20,14 @@ from mycli.compat import WIN
 from mycli.packages.prompt_utils import confirm_destructive_query
 from mycli.packages.special.delimitercommand import DelimiterCommand
 from mycli.packages.special.favoritequeries import FavoriteQueries
+from mycli.packages.special.main import COMMANDS as SPECIAL_COMMANDS
 from mycli.packages.special.main import ArgType, special_command
+from mycli.packages.special.main import execute as special_execute
 from mycli.packages.special.utils import handle_cd_command
+from mycli.packages.sqlresult import SQLResult
+
+sqlparse.engine.grouping.MAX_GROUPING_DEPTH = None  # type: ignore[assignment]
+sqlparse.engine.grouping.MAX_GROUPING_TOKENS = None  # type: ignore[assignment]
 
 TIMING_ENABLED = False
 use_expanded_output = False
@@ -38,6 +45,7 @@ PIPE_ONCE: dict[str, Any] = {
 }
 delimiter_command = DelimiterCommand()
 favoritequeries = FavoriteQueries(ConfigObj())
+DESTRUCTIVE_KEYWORDS: list[str] = []
 
 
 def set_favorite_queries(config):
@@ -68,15 +76,20 @@ def is_show_favorite_query() -> bool:
     return SHOW_FAVORITE_QUERY
 
 
+def set_destructive_keywords(val: list[str]) -> None:
+    global DESTRUCTIVE_KEYWORDS
+    DESTRUCTIVE_KEYWORDS = val
+
+
 @special_command(
     "pager",
-    "\\P [command]",
-    "Set PAGER. Print the query results via PAGER.",
+    "pager [command]",
+    "Set pager to [command]. Print query results via pager.",
     arg_type=ArgType.PARSED_QUERY,
     aliases=["\\P"],
     case_sensitive=True,
 )
-def set_pager(arg: str, **_) -> list[tuple]:
+def set_pager(arg: str, **_) -> list[SQLResult]:
     if arg:
         os.environ["PAGER"] = arg
         msg = f"PAGER set to {arg}."
@@ -89,22 +102,22 @@ def set_pager(arg: str, **_) -> list[tuple]:
             msg = "Pager enabled."
         set_pager_enabled(True)
 
-    return [(None, None, None, msg)]
+    return [SQLResult(status=msg)]
 
 
-@special_command("nopager", "\\n", "Disable pager, print to stdout.", arg_type=ArgType.NO_QUERY, aliases=["\\n"], case_sensitive=True)
-def disable_pager() -> list[tuple]:
+@special_command("nopager", "nopager", "Disable pager; print to stdout.", arg_type=ArgType.NO_QUERY, aliases=["\\n"], case_sensitive=True)
+def disable_pager() -> list[SQLResult]:
     set_pager_enabled(False)
-    return [(None, None, None, "Pager disabled.")]
+    return [SQLResult(status="Pager disabled.")]
 
 
-@special_command("\\timing", "\\t", "Toggle timing of commands.", arg_type=ArgType.NO_QUERY, aliases=["\\t"], case_sensitive=True)
-def toggle_timing() -> list[tuple]:
+@special_command("\\timing", "\\timing", "Toggle timing of queries.", arg_type=ArgType.NO_QUERY, aliases=["\\t"], case_sensitive=True)
+def toggle_timing() -> list[SQLResult]:
     global TIMING_ENABLED
     TIMING_ENABLED = not TIMING_ENABLED
     message = "Timing is "
     message += "on." if TIMING_ENABLED else "off."
-    return [(None, None, None, message)]
+    return [SQLResult(status=message)]
 
 
 def is_timing_enabled() -> bool:
@@ -139,11 +152,16 @@ def editor_command(command: str) -> bool:
     """
     # It is possible to have `\e filename` or `SELECT * FROM \e`. So we check
     # for both conditions.
-    return command.strip().endswith("\\e") or command.strip().startswith("\\e")
+    return (
+        command.strip().endswith("\\e")
+        or command.strip().startswith("\\e ")
+        or command.strip().endswith("\\edit")
+        or command.strip().startswith("\\edit ")
+    )
 
 
 def get_filename(sql: str) -> str | None:
-    if sql.strip().startswith("\\e"):
+    if sql.strip().startswith("\\e ") or sql.strip().startswith("\\edit "):
         command, _, filename = sql.partition(" ")
         return filename.strip() or None
     else:
@@ -157,7 +175,7 @@ def get_editor_query(sql: str) -> str:
     # The reason we can't simply do .strip('\e') is that it strips characters,
     # not a substring. So it'll strip "e" in the end of the sql also!
     # Ex: "select * from style\e" -> "select * from styl".
-    pattern = re.compile(r"(^\\e|\\e$)")
+    pattern = re.compile(r"(\\e$|\\edit$)")
     while pattern.search(sql):
         sql = pattern.sub("", sql)
 
@@ -182,7 +200,7 @@ def open_external_editor(filename: str | None = None, sql: str | None = None) ->
                 query = f.read()
         except IOError:
             message = f'Error reading file: {filename}'
-        return (query, message)
+        return (query.rstrip('\n'), message)
 
     # Populate the editor buffer with the partial sql (if available) and a
     # placeholder comment.
@@ -249,11 +267,9 @@ def set_redirect(command_part: str | None, file_operator_part: str | None, file_
 
 
 @special_command("\\f", "\\f [name [args..]]", "List or execute favorite queries.", arg_type=ArgType.PARSED_QUERY, case_sensitive=True)
-def execute_favorite_query(cur: Cursor, arg: str, **_) -> Generator[tuple, None, None]:
-    """Returns (title, rows, headers, status)"""
+def execute_favorite_query(cur: Cursor, arg: str, **_) -> Generator[SQLResult, None, None]:
     if arg == "":
-        for result in list_favorite_queries():
-            yield result
+        yield from list_favorite_queries()
 
     # Parse out favorite name and optional substitution parameters
     name, _separator, arg_str = arg.partition(" ")
@@ -262,35 +278,45 @@ def execute_favorite_query(cur: Cursor, arg: str, **_) -> Generator[tuple, None,
     query = FavoriteQueries.instance.get(name)
     if query is None:
         message = f"No favorite query: {name}"
-        yield (None, None, None, message)
+        yield SQLResult(status=message)
     else:
         query, arg_error = subst_favorite_query_args(query, args)
         if query is None:
-            yield (None, None, None, arg_error)
+            yield SQLResult(status=arg_error)
         else:
             for sql in sqlparse.split(query):
                 sql = sql.rstrip(";")
-                title = f"> {sql}" if is_show_favorite_query() else None
-                cur.execute(sql)
-                if cur.description:
-                    headers = [x[0] for x in cur.description]
-                    yield (title, cur, headers, None)
+                preamble = f"> {sql}" if is_show_favorite_query() else None
+                is_special = False
+                for special in SPECIAL_COMMANDS:
+                    if sql.lower().startswith(special.lower()):
+                        is_special = True
+                        break
+                if is_special:
+                    for result in special_execute(cur, sql):
+                        result.preamble = preamble
+                        # special_execute() already returns a SQLResult
+                        yield result
                 else:
-                    yield (title, None, None, None)
+                    cur.execute(sql)
+                    if cur.description:
+                        header = [x[0] for x in cur.description]
+                        yield SQLResult(preamble=preamble, header=header, rows=cur)
+                    else:
+                        yield SQLResult(preamble=preamble)
 
 
-def list_favorite_queries() -> list[tuple]:
-    """List of all favorite queries.
-    Returns (title, rows, headers, status)"""
+def list_favorite_queries() -> list[SQLResult]:
+    """List of all favorite queries."""
 
-    headers = ["Name", "Query"]
+    header = ["Name", "Query"]
     rows = [(r, FavoriteQueries.instance.get(r)) for r in FavoriteQueries.instance.list()]
 
     if not rows:
         status = "\nNo favorite queries found." + FavoriteQueries.instance.usage
     else:
         status = ""
-    return [("", rows, headers, status)]
+    return [SQLResult(header=header, rows=rows, status=status)]
 
 
 def subst_favorite_query_args(query: str, args: list[str]) -> list[str | None]:
@@ -309,52 +335,51 @@ def subst_favorite_query_args(query: str, args: list[str]) -> list[str | None]:
     return [query, None]
 
 
-@special_command("\\fs", "\\fs name query", "Save a favorite query.")
-def save_favorite_query(arg: str, **_) -> list[tuple]:
-    """Save a new favorite query.
-    Returns (title, rows, headers, status)"""
+@special_command("\\fs", "\\fs <name> <query>", "Save a favorite query.")
+def save_favorite_query(arg: str, **_) -> list[SQLResult]:
+    """Save a new favorite query."""
 
     usage = "Syntax: \\fs name query.\n\n" + FavoriteQueries.instance.usage
     if not arg:
-        return [(None, None, None, usage)]
+        return [SQLResult(status=usage)]
 
     name, _separator, query = arg.partition(" ")
 
     # If either name or query is missing then print the usage and complain.
     if (not name) or (not query):
-        return [(None, None, None, usage + "Err: Both name and query are required.")]
+        return [SQLResult(status=f"{usage} Err: Both name and query are required.")]
 
     FavoriteQueries.instance.save(name, query)
-    return [(None, None, None, "Saved.")]
+    return [SQLResult(status="Saved.")]
 
 
-@special_command("\\fd", "\\fd [name]", "Delete a favorite query.")
-def delete_favorite_query(arg: str, **_) -> list[tuple]:
+@special_command("\\fd", "\\fd <name>", "Delete a favorite query.")
+def delete_favorite_query(arg: str, **_) -> list[SQLResult]:
     """Delete an existing favorite query."""
     usage = "Syntax: \\fd name.\n\n" + FavoriteQueries.instance.usage
     if not arg:
-        return [(None, None, None, usage)]
+        return [SQLResult(status=usage)]
 
     status = FavoriteQueries.instance.delete(arg)
 
-    return [(None, None, None, status)]
+    return [SQLResult(status=status)]
 
 
-@special_command("system", "system [command]", "Execute a system shell commmand.")
-def execute_system_command(arg: str, **_) -> list[tuple]:
+@special_command("system", "system <command>", "Execute a system shell commmand.")
+def execute_system_command(arg: str, **_) -> list[SQLResult]:
     """Execute a system shell command."""
     usage = "Syntax: system [command].\n"
 
     if not arg:
-        return [(None, None, None, usage)]
+        return [SQLResult(status=usage)]
 
     try:
         command = arg.strip()
         if command.startswith("cd"):
             ok, error_message = handle_cd_command(arg)
             if not ok:
-                return [(None, None, None, error_message)]
-            return [(None, None, None, "")]
+                return [SQLResult(status=error_message)]
+            return [SQLResult(status="")]
 
         args = arg.split(" ")
         process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
@@ -364,9 +389,9 @@ def execute_system_command(arg: str, **_) -> list[tuple]:
         encoding = locale.getpreferredencoding(False)
         response_str = response.decode(encoding)
 
-        return [(None, None, None, response_str)]
+        return [SQLResult(status=response_str)]
     except OSError as e:
-        return [(None, None, None, f"OSError: {e.strerror}")]
+        return [SQLResult(status=f"OSError: {e.strerror}")]
 
 
 def parseargfile(arg: str) -> tuple[str, str]:
@@ -383,8 +408,8 @@ def parseargfile(arg: str) -> tuple[str, str]:
     return (os.path.expanduser(filename), mode)
 
 
-@special_command("tee", "tee [-o] filename", "Append all results to an output file (overwrite using -o).")
-def set_tee(arg: str, **_) -> list[tuple]:
+@special_command("tee", "tee [-o] <filename>", "Append all results to an output file (overwrite using -o).")
+def set_tee(arg: str, **_) -> list[SQLResult]:
     global tee_file
 
     try:
@@ -392,7 +417,7 @@ def set_tee(arg: str, **_) -> list[tuple]:
     except (IOError, OSError) as e:
         raise OSError(f"Cannot write to file '{e.filename}': {e.strerror}") from e
 
-    return [(None, None, None, "")]
+    return [SQLResult(status="")]
 
 
 def close_tee() -> None:
@@ -403,21 +428,23 @@ def close_tee() -> None:
 
 
 @special_command("notee", "notee", "Stop writing results to an output file.")
-def no_tee(arg: str, **_) -> list[tuple]:
+def no_tee(arg: str, **_) -> list[SQLResult]:
     close_tee()
-    return [(None, None, None, "")]
+    return [SQLResult(status="")]
 
 
-def write_tee(output: str) -> None:
+def write_tee(output: str | ANSI | FormattedText, nl: bool = True) -> None:
     global tee_file
-    if tee_file:
-        click.echo(output, file=tee_file, nl=False)
-        click.echo("\n", file=tee_file, nl=False)
-        tee_file.flush()
+    if not tee_file:
+        return
+    click.echo(to_plain_text(output), file=tee_file, nl=False)
+    if nl:
+        click.echo('\n', file=tee_file, nl=False)
+    tee_file.flush()
 
 
-@special_command("\\once", "\\o [-o] filename", "Append next result to an output file (overwrite using -o).", aliases=["\\o"])
-def set_once(arg: str, **_) -> list[tuple]:
+@special_command("\\once", "\\once [-o] <filename>", "Append next result to an output file (overwrite using -o).", aliases=["\\o"])
+def set_once(arg: str, **_) -> list[SQLResult]:
     global once_file, written_to_once_file
 
     try:
@@ -426,7 +453,7 @@ def set_once(arg: str, **_) -> list[tuple]:
         raise OSError(f"Cannot write to file '{e.filename}': {e.strerror}") from e
     written_to_once_file = False
 
-    return [(None, None, None, "")]
+    return [SQLResult(status="")]
 
 
 def is_redirected() -> bool:
@@ -469,8 +496,8 @@ def _run_post_redirect_hook(post_redirect_command: str, filename: str) -> None:
         raise OSError(f"Redirect post hook failed: {e}") from e
 
 
-@special_command("\\pipe_once", "\\| command", "Send next result to a subprocess.", aliases=["\\|"])
-def set_pipe_once(arg: str, **_) -> list[tuple]:
+@special_command("\\pipe_once", "\\pipe_once <command>", "Send next result to a subprocess.", aliases=["\\|"])
+def set_pipe_once(arg: str, **_) -> list[SQLResult]:
     if not arg:
         raise OSError("pipe_once requires a command")
     if WIN:
@@ -488,7 +515,7 @@ def set_pipe_once(arg: str, **_) -> list[tuple]:
         encoding="UTF-8",
         universal_newlines=True,
     )
-    return [(None, None, None, "")]
+    return [SQLResult(status="")]
 
 
 def write_pipe_once(line: str) -> None:
@@ -528,15 +555,15 @@ def flush_pipe_once_if_written(post_redirect_command: str) -> None:
     PIPE_ONCE['stdout_mode'] = None
 
 
-@special_command("watch", "watch [seconds] [-c] query", "Executes the query every [seconds] seconds (by default 5).")
-def watch_query(arg: str, **kwargs) -> Generator[tuple, None, None]:
+@special_command("watch", "watch [seconds] [-c] <query>", "Execute query every [seconds] seconds (5 by default).")
+def watch_query(arg: str, **kwargs) -> Generator[SQLResult, None, None]:
     usage = """Syntax: watch [seconds] [-c] query.
     * seconds: The interval at the query will be repeated, in seconds.
                By default 5.
     * -c: Clears the screen between every iteration.
 """
     if not arg:
-        yield (None, None, None, usage)
+        yield SQLResult(status=usage)
         return
     seconds = 5.0
     clear_screen = False
@@ -545,7 +572,7 @@ def watch_query(arg: str, **kwargs) -> Generator[tuple, None, None]:
         arg = arg.strip()
         if not arg:
             # Oops, we parsed all the arguments without finding a statement
-            yield (None, None, None, usage)
+            yield SQLResult(status=usage)
             return
         (left_arg, _, right_arg) = arg.partition(" ")
         arg = right_arg
@@ -558,7 +585,7 @@ def watch_query(arg: str, **kwargs) -> Generator[tuple, None, None]:
             clear_screen = True
             continue
         statement = f"{left_arg} {arg}"
-    destructive_prompt = confirm_destructive_query(statement)
+    destructive_prompt = confirm_destructive_query(DESTRUCTIVE_KEYWORDS, statement)
     if destructive_prompt is False:
         click.secho("Wise choice!")
         return
@@ -574,13 +601,17 @@ def watch_query(arg: str, **kwargs) -> Generator[tuple, None, None]:
             # Somewhere in the code the pager its activated after every yield,
             # so we disable it in every iteration
             set_pager_enabled(False)
-            for sql, title in sql_list:
+            for sql, preamble in sql_list:
                 cur.execute(sql)
+                command: dict[str, str | float] = {
+                    "name": "watch",
+                    "seconds": seconds,
+                }
                 if cur.description:
-                    headers = [x[0] for x in cur.description]
-                    yield (title, cur, headers, None)
+                    header = [x[0] for x in cur.description]
+                    yield SQLResult(preamble=preamble, header=header, rows=cur, command=command)
                 else:
-                    yield (title, None, None, None)
+                    yield SQLResult(preamble=preamble, command=command)
             sleep(seconds)
         except KeyboardInterrupt:
             # This prints the Ctrl-C character in its own line, which prevents
@@ -591,8 +622,8 @@ def watch_query(arg: str, **kwargs) -> Generator[tuple, None, None]:
             set_pager_enabled(old_pager_enabled)
 
 
-@special_command("delimiter", None, "Change SQL delimiter.")
-def set_delimiter(arg: str, **_) -> list[tuple]:
+@special_command("delimiter", "delimiter <string>", "Change end-of-statement delimiter.")
+def set_delimiter(arg: str, **_) -> list[SQLResult]:
     return delimiter_command.set(arg)
 
 
@@ -601,5 +632,4 @@ def get_current_delimiter() -> str:
 
 
 def split_queries(input_str: str) -> Generator[str, None, None]:
-    for query in delimiter_command.queries_iter(input_str):
-        yield query
+    yield from delimiter_command.queries_iter(input_str)
