@@ -14,6 +14,8 @@ from textwrap import dedent
 from behave import then, when
 import wrappers
 
+from test.utils import TEMPFILE_PREFIX
+
 
 @when("we run dbcli")
 def step_run_cli(context):
@@ -55,10 +57,10 @@ def step_send_help(context):
 
 @when("we send source command")
 def step_send_source_command(context):
-    with tempfile.NamedTemporaryFile() as f:
+    with tempfile.NamedTemporaryFile(prefix=TEMPFILE_PREFIX) as f:
         f.write(b"\\?")
         f.flush()
-        context.cli.sendline("\\. {0}".format(f.name))
+        context.cli.sendline(f"\\. {f.name}")
         wrappers.expect_exact(context, context.conf["pager_boundary"] + "\r\n", timeout=5)
 
 
@@ -71,19 +73,22 @@ def step_check_application_name(context):
 
 @then("we see found")
 def step_see_found(context):
+    expected = (
+        dedent(
+            """
+            +-------+\r
+            | found |\r
+            +-------+\r
+            | found |\r
+            +-------+
+            """
+        ).strip()
+        + '\r\n\r\n'
+    )
+
     wrappers.expect_exact(
         context,
-        context.conf["pager_boundary"]
-        + "\r"
-        + dedent("""
-            +-------+\r
-            | found |\r
-            +-------+\r
-            | found |\r
-            +-------+\r
-            \r
-        """)
-        + context.conf["pager_boundary"],
+        context.conf["pager_boundary"] + '\r\n' + expected + context.conf["pager_boundary"],
         timeout=5,
     )
 
@@ -94,19 +99,22 @@ def step_see_date(context):
     # such as running near midnight when the test database has
     # a different TZ setting than the system.
     date_str = datetime.datetime.now().strftime("%Y-%m-%d")
-    wrappers.expect_exact(
-        context,
-        context.conf["pager_boundary"]
-        + "\r"
-        + dedent(f"""
+    expected = (
+        dedent(
+            f"""
             +------------+\r
             | dt         |\r
             +------------+\r
             | {date_str} |\r
-            +------------+\r
-            \r
-        """)
-        + context.conf["pager_boundary"],
+            +------------+
+            """
+        ).strip()
+        + '\r\n\r\n'
+    )
+
+    wrappers.expect_exact(
+        context,
+        context.conf["pager_boundary"] + '\r\n' + expected + context.conf["pager_boundary"],
         timeout=5,
     )
 
