@@ -21,7 +21,6 @@ def test_select_suggests_cols_with_visible_table_scope():
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [(None, "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -31,7 +30,6 @@ def test_select_suggests_cols_with_qualified_table_scope():
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [("sch", "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -55,7 +53,6 @@ def test_where_suggests_columns_functions(expression):
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [(None, "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -67,7 +64,6 @@ def test_where_equals_suggests_enum_values_first():
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [(None, "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -84,7 +80,6 @@ def test_where_in_suggests_columns(expression):
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [(None, "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -95,7 +90,6 @@ def test_where_equals_any_suggests_columns_or_keywords():
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [(None, "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -120,7 +114,6 @@ def test_select_suggests_cols_and_funcs():
         {"type": "alias", "aliases": []},
         {"type": "column", "tables": []},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -193,7 +186,6 @@ def test_col_comma_suggests_cols():
         {"type": "alias", "aliases": ["tbl"]},
         {"type": "column", "tables": [(None, "tbl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -236,7 +228,6 @@ def test_partially_typed_col_name_suggests_col_names():
         {"type": "alias", "aliases": ["tabl"]},
         {"type": "column", "tables": [(None, "tabl", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -331,7 +322,6 @@ def test_sub_select_col_name_completion():
         {"type": "alias", "aliases": ["abc"]},
         {"type": "column", "tables": [(None, "abc", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -484,7 +474,6 @@ def test_2_statements_2nd_current():
         {"type": "alias", "aliases": ["b"]},
         {"type": "column", "tables": [(None, "b", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
     # Should work even if first statement is invalid
@@ -509,7 +498,6 @@ def test_2_statements_1st_current():
         {"type": "alias", "aliases": ["a"]},
         {"type": "column", "tables": [(None, "a", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -526,7 +514,6 @@ def test_3_statements_2nd_current():
         {"type": "alias", "aliases": ["b"]},
         {"type": "column", "tables": [(None, "b", None)]},
         {"type": "function", "schema": []},
-        {"type": "keyword"},
     ])
 
 
@@ -606,7 +593,7 @@ def test_after_as(expression):
 )
 def test_source_is_file(expression):
     # "source" has to be registered by hand because that usually happens inside MyCLI in mycli/main.py
-    special.register_special_command(..., 'source', '\\. filename', 'Execute commands from file.', aliases=['\\.'])
+    special.register_special_command(..., 'source', '\\. <filename>', 'Execute commands from file.', aliases=['\\.'])
     suggestions = suggest_type(expression, expression)
     assert suggestions == [{"type": "file_name"}]
 
