@@ -6,7 +6,7 @@ from io import BytesIO, StringIO, TextIOWrapper
 import os
 import struct
 import sys
-import tempfile
+from tempfile import NamedTemporaryFile
 
 import pytest
 
@@ -18,6 +18,7 @@ from mycli.config import (
     str_to_bool,
     strip_matching_quotes,
 )
+from test.utils import TEMPFILE_PREFIX
 
 LOGIN_PATH_FILE = os.path.abspath(os.path.join(os.path.dirname(__file__), "mylogin.cnf"))
 
@@ -89,18 +90,13 @@ def test_corrupted_pad():
     assert "user" not in contents
 
 
-def test_get_mylogin_cnf_path():
+def test_get_mylogin_cnf_path(monkeypatch):
     """Tests that the path for .mylogin.cnf is detected."""
-    original_env = None
-    if "MYSQL_TEST_LOGIN_FILE" in os.environ:
-        original_env = os.environ.pop("MYSQL_TEST_LOGIN_FILE")
+    monkeypatch.delenv('MYSQL_TEST_LOGIN_FILE', raising=False)
     is_windows = sys.platform == "win32"
 
     login_cnf_path = get_mylogin_cnf_path()
 
-    if original_env is not None:
-        os.environ["MYSQL_TEST_LOGIN_FILE"] = original_env
-
     if login_cnf_path is not None:
         assert login_cnf_path.endswith(".mylogin.cnf")
 
@@ -111,21 +107,20 @@ def test_get_mylogin_cnf_path():
             assert login_cnf_path.startswith(home_dir)
 
 
-def test_alternate_get_mylogin_cnf_path():
+def test_alternate_get_mylogin_cnf_path(monkeypatch):
     """Tests that the alternate path for .mylogin.cnf is detected."""
-    original_env = None
-    if "MYSQL_TEST_LOGIN_FILE" in os.environ:
-        original_env = os.environ.pop("MYSQL_TEST_LOGIN_FILE")
 
-    _, temp_path = tempfile.mkstemp()
-    os.environ["MYSQL_TEST_LOGIN_FILE"] = temp_path
+    with NamedTemporaryFile(prefix=TEMPFILE_PREFIX, mode='w', delete=False) as login_file:
+        monkeypatch.setenv('MYSQL_TEST_LOGIN_FILE', login_file.name)
+        login_cnf_path = get_mylogin_cnf_path()
 
-    login_cnf_path = get_mylogin_cnf_path()
-
-    if original_env is not None:
-        os.environ["MYSQL_TEST_LOGIN_FILE"] = original_env
-
-    assert temp_path == login_cnf_path
+    try:
+        assert login_file.name == login_cnf_path
+    except AssertionError as e:
+        assert AssertionError(e)
+    finally:
+        if os.path.exists(login_file.name):
+            os.remove(login_file.name)
 
 
 def test_str_to_bool():
@@ -171,16 +166,16 @@ def test_strip_quotes_with_matching_quotes():
     """Test that a string with matching quotes is unquoted."""
 
     s = "May the force be with you."
-    assert s == strip_matching_quotes('"{}"'.format(s))
-    assert s == strip_matching_quotes("'{}'".format(s))
+    assert s == strip_matching_quotes(f'"{s}"')
+    assert s == strip_matching_quotes(f"'{s}'")
 
 
 def test_strip_quotes_with_unmatching_quotes():
     """Test that a string with unmatching quotes is not unquoted."""
 
     s = "May the force be with you."
-    assert '"' + s == strip_matching_quotes('"{}'.format(s))
-    assert s + "'" == strip_matching_quotes("{}'".format(s))
+    assert '"' + s == strip_matching_quotes(f'"{s}')
+    assert s + "'" == strip_matching_quotes(f"{s}'")
 
 
 def test_strip_quotes_with_empty_string():
