@@ -1,5 +1,6 @@
 # type: ignore
 
+from datetime import time
 import os
 
 import pymysql
@@ -9,11 +10,26 @@ from mycli.sqlexecute import ServerInfo, ServerSpecies
 from test.utils import dbtest, is_expanded_output, run, set_expanded_output
 
 
-def assert_result_equal(result, title=None, rows=None, headers=None, status=None, auto_status=True, assert_contains=False):
+def assert_result_equal(
+    result,
+    preamble=None,
+    header=None,
+    rows=None,
+    status=None,
+    postamble=None,
+    auto_status=True,
+    assert_contains=False,
+):
     """Assert that an sqlexecute.run() result matches the expected values."""
     if status is None and auto_status and rows:
-        status = "{} row{} in set".format(len(rows), "s" if len(rows) > 1 else "")
-    fields = {"title": title, "rows": rows, "headers": headers, "status": status}
+        status = f"{len(rows)} row{'s' if len(rows) > 1 else ''} in set"
+    fields = {
+        "preamble": preamble,
+        "header": header,
+        "rows": rows,
+        "postamble": postamble,
+        "status": status,
+    }
 
     if assert_contains:
         # Do a loose match on the results using the *in* operator.
@@ -25,13 +41,43 @@ def assert_result_equal(result, title=None, rows=None, headers=None, status=None
         assert result == [fields]
 
 
+@dbtest
+def test_timediff_negative_value(executor):
+    sql = "select timediff('2020-11-11 01:01:01', '2020-11-11 01:02:01')"
+    result = run(executor, sql)
+    # negative value comes back as str
+    assert result[0]["rows"][0][0] == "-00:01:00"
+
+
+@dbtest
+def test_timediff_positive_value(executor):
+    sql = "select timediff('2020-11-11 01:02:01', '2020-11-11 01:01:01')"
+    result = run(executor, sql)
+    # positive value comes back as datetime.time
+    assert result[0]["rows"][0][0] == time(0, 1)
+
+
+@dbtest
+def test_get_result_status_without_warning(executor):
+    sql = "select 1"
+    result = run(executor, sql)
+    assert result[0]["status"] == "1 row in set"
+
+
+@dbtest
+def test_get_result_status_with_warning(executor):
+    sql = "SELECT 1 + '0 foo'"
+    result = run(executor, sql)
+    assert result[0]["status"] == "1 row in set, 1 warning"
+
+
 @dbtest
 def test_conn(executor):
     run(executor, """create table test(a text)""")
     run(executor, """insert into test values('abc')""")
     results = run(executor, """select * from test""")
 
-    assert_result_equal(results, headers=["a"], rows=[("abc",)])
+    assert_result_equal(results, header=["a"], rows=[("abc",)])
 
 
 @dbtest
@@ -40,7 +86,7 @@ def test_bools(executor):
     run(executor, """insert into test values(True)""")
     results = run(executor, """select * from test""")
 
-    assert_result_equal(results, headers=["a"], rows=[(1,)])
+    assert_result_equal(results, header=["a"], rows=[(1,)])
 
 
 @dbtest
@@ -55,7 +101,7 @@ def test_binary(executor):
         b"\xac\xdeC@"
     )
 
-    assert_result_equal(results, headers=["geom"], rows=[(geom,)])
+    assert_result_equal(results, header=["geom"], rows=[(geom,)])
 
 
 @dbtest
@@ -94,7 +140,7 @@ def test_unicode_support_in_output(executor):
 
     # See issue #24, this raises an exception without proper handling
     results = run(executor, "select * from unicodechars")
-    assert_result_equal(results, headers=["t"], rows=[("é",)])
+    assert_result_equal(results, header=["t"], rows=[("é",)])
 
 
 @dbtest
@@ -102,8 +148,8 @@ def test_multiple_queries_same_line(executor):
     results = run(executor, "select 'foo'; select 'bar'")
 
     expected = [
-        {"title": None, "headers": ["foo"], "rows": [("foo",)], "status": "1 row in set"},
-        {"title": None, "headers": ["bar"], "rows": [("bar",)], "status": "1 row in set"},
+        {"preamble": None, "header": ["foo"], "rows": [("foo",)], "postamble": None, "status": "1 row in set"},
+        {"preamble": None, "header": ["bar"], "rows": [("bar",)], "postamble": None, "status": "1 row in set"},
     ]
     assert expected == results
 
@@ -127,10 +173,10 @@ def test_favorite_query(executor):
     assert_result_equal(results, status="Saved.")
 
     results = run(executor, "\\f test-a")
-    assert_result_equal(results, title="> select * from test where a like 'a%'", headers=["a"], rows=[("abc",)], auto_status=False)
+    assert_result_equal(results, preamble="> select * from test where a like 'a%'", header=["a"], rows=[("abc",)], auto_status=False)
 
     results = run(executor, "\\fd test-a")
-    assert_result_equal(results, status="test-a: Deleted")
+    assert_result_equal(results, status="test-a: Deleted.")
 
 
 @dbtest
@@ -146,13 +192,13 @@ def test_favorite_query_multiple_statement(executor):
 
     results = run(executor, "\\f test-ad")
     expected = [
-        {"title": "> select * from test where a like 'a%'", "headers": ["a"], "rows": [("abc",)], "status": None},
-        {"title": "> select * from test where a like 'd%'", "headers": ["a"], "rows": [("def",)], "status": None},
+        {"preamble": "> select * from test where a like 'a%'", "header": ["a"], "rows": [("abc",)], "postamble": None, "status": None},
+        {"preamble": "> select * from test where a like 'd%'", "header": ["a"], "rows": [("def",)], "postamble": None, "status": None},
     ]
     assert expected == results
 
     results = run(executor, "\\fd test-ad")
-    assert_result_equal(results, status="test-ad: Deleted")
+    assert_result_equal(results, status="test-ad: Deleted.")
 
 
 @dbtest
@@ -167,12 +213,12 @@ def test_favorite_query_expanded_output(executor):
 
     results = run(executor, "\\f test-ae \\G")
     assert is_expanded_output() is True
-    assert_result_equal(results, title="> select * from test", headers=["a"], rows=[("abc",)], auto_status=False)
+    assert_result_equal(results, preamble="> select * from test", header=["a"], rows=[("abc",)], auto_status=False)
 
     set_expanded_output(False)
 
     results = run(executor, "\\fd test-ae")
-    assert_result_equal(results, status="test-ae: Deleted")
+    assert_result_equal(results, status="test-ae: Deleted.")
 
 
 @dbtest
@@ -185,13 +231,38 @@ def test_collapsed_output_special_command(executor):
 @dbtest
 def test_special_command(executor):
     results = run(executor, "\\?")
-    assert_result_equal(results, rows=("quit", "\\q", "Quit."), headers="Command", assert_contains=True, auto_status=False)
+    assert_result_equal(results, rows=("quit", "\\q", "quit", "Quit."), header="Command", assert_contains=True, auto_status=False)
 
 
 @dbtest
 def test_cd_command_without_a_folder_name(executor):
     results = run(executor, "system cd")
-    assert_result_equal(results, status="No folder name was provided.")
+    assert_result_equal(results, status="Exactly one directory name must be provided.")
+
+
+@dbtest
+def test_cd_command_with_one_nonexistent_folder_name(executor):
+    results = run(executor, 'system cd nonexistent_folder_name')
+    assert_result_equal(results, status='No such file or directory')
+
+
+@dbtest
+def test_cd_command_with_one_real_folder_name(executor):
+    results = run(executor, 'system cd screenshots')
+    # todo would be better to capture stderr but there was a problem with capsys
+    assert results[0]['status'] == ''
+
+
+@dbtest
+def test_cd_command_with_two_folder_names(executor):
+    results = run(executor, "system cd one two")
+    assert_result_equal(results, status='Exactly one directory name must be provided.')
+
+
+@dbtest
+def test_cd_command_unbalanced(executor):
+    results = run(executor, "system cd 'one")
+    assert_result_equal(results, status='Cannot parse cd command.')
 
 
 @dbtest
@@ -208,21 +279,21 @@ def test_system_command_output(executor):
     eol = os.linesep
     test_dir = os.path.abspath(os.path.dirname(__file__))
     test_file_path = os.path.join(test_dir, "test.txt")
-    results = run(executor, "system cat {0}".format(test_file_path))
+    results = run(executor, f"system cat {test_file_path}")
     assert_result_equal(results, status=f"mycli rocks!{eol}")
 
 
 @dbtest
 def test_cd_command_current_dir(executor):
     test_path = os.path.abspath(os.path.dirname(__file__))
-    run(executor, "system cd {0}".format(test_path))
+    run(executor, f"system cd {test_path}")
     assert os.getcwd() == test_path
 
 
 @dbtest
 def test_unicode_support(executor):
     results = run(executor, "SELECT '日本語' AS japanese;")
-    assert_result_equal(results, headers=["japanese"], rows=[("日本語",)])
+    assert_result_equal(results, header=["japanese"], rows=[("日本語",)])
 
 
 @dbtest
@@ -230,7 +301,7 @@ def test_timestamp_null(executor):
     run(executor, """create table ts_null(a timestamp null)""")
     run(executor, """insert into ts_null values(null)""")
     results = run(executor, """select * from ts_null""")
-    assert_result_equal(results, headers=["a"], rows=[(None,)])
+    assert_result_equal(results, header=["a"], rows=[(None,)])
 
 
 @dbtest
@@ -238,7 +309,7 @@ def test_datetime_null(executor):
     run(executor, """create table dt_null(a datetime null)""")
     run(executor, """insert into dt_null values(null)""")
     results = run(executor, """select * from dt_null""")
-    assert_result_equal(results, headers=["a"], rows=[(None,)])
+    assert_result_equal(results, header=["a"], rows=[(None,)])
 
 
 @dbtest
@@ -246,7 +317,7 @@ def test_date_null(executor):
     run(executor, """create table date_null(a date null)""")
     run(executor, """insert into date_null values(null)""")
     results = run(executor, """select * from date_null""")
-    assert_result_equal(results, headers=["a"], rows=[(None,)])
+    assert_result_equal(results, header=["a"], rows=[(None,)])
 
 
 @dbtest
@@ -254,7 +325,7 @@ def test_time_null(executor):
     run(executor, """create table time_null(a time null)""")
     run(executor, """insert into time_null values(null)""")
     results = run(executor, """select * from time_null""")
-    assert_result_equal(results, headers=["a"], rows=[(None,)])
+    assert_result_equal(results, header=["a"], rows=[(None,)])
 
 
 @dbtest
@@ -268,8 +339,8 @@ def test_multiple_results(executor):
 
     results = run(executor, "call dmtest;")
     expected = [
-        {"title": None, "rows": [(1,)], "headers": ["1"], "status": "1 row in set"},
-        {"title": None, "rows": [(2,)], "headers": ["2"], "status": "1 row in set"},
+        {"preamble": None, "header": ["1"], "rows": [(1,)], "postamble": None, "status": "1 row in set"},
+        {"preamble": None, "header": ["2"], "rows": [(2,)], "postamble": None, "status": "1 row in set"},
     ]
     assert results == expected
 
