@@ -11,18 +11,20 @@ import pytest
 
 from mycli.main import special
 
+DATABASE = "mycli_test_db"
 PASSWORD = os.getenv("PYTEST_PASSWORD")
 USER = os.getenv("PYTEST_USER", "root")
 HOST = os.getenv("PYTEST_HOST", "localhost")
 PORT = int(os.getenv("PYTEST_PORT", "3306"))
-CHARSET = os.getenv("PYTEST_CHARSET", "utf8")
+CHARACTER_SET = os.getenv("PYTEST_CHARSET", "utf8mb4")
 SSH_USER = os.getenv("PYTEST_SSH_USER", None)
 SSH_HOST = os.getenv("PYTEST_SSH_HOST", None)
 SSH_PORT = int(os.getenv("PYTEST_SSH_PORT", "22"))
+TEMPFILE_PREFIX = 'mycli_test_suite_'
 
 
 def db_connection(dbname=None):
-    conn = pymysql.connect(user=USER, host=HOST, port=PORT, database=dbname, password=PASSWORD, charset=CHARSET, local_infile=False)
+    conn = pymysql.connect(user=USER, host=HOST, port=PORT, database=dbname, password=PASSWORD, charset=CHARACTER_SET, local_infile=False)
     conn.autocommit = True
     return conn
 
@@ -47,13 +49,19 @@ def create_db(dbname):
 
 def run(executor, sql, rows_as_list=True):
     """Return string output for the sql to be run."""
-    result = []
+    results = []
 
-    for title, rows, headers, status in executor.run(sql):
-        rows = list(rows) if (rows_as_list and rows) else rows
-        result.append({"title": title, "rows": rows, "headers": headers, "status": status})
+    for result in executor.run(sql):
+        rows = list(result.rows) if (rows_as_list and result.rows) else result.rows
+        results.append({
+            "preamble": result.preamble,
+            "header": result.header,
+            "rows": rows,
+            "postamble": result.postamble,
+            "status": result.status,
+        })
 
-    return result
+    return results
 
 
 def set_expanded_output(is_expanded):
